import { DEFAULT_OVERVIEW_IMAGE_SIZE } from '@/constants/project';
import { currentOverviewImageSelector } from '@/redux/project/project.selectors';
import { useSelector } from 'react-redux';
import { ImageContainerSize, OverviewImageSize } from '../OverviewImageModal.types';

interface UseOverviewImageArgs {
  containerRect?: DOMRect;
}

interface UseOverviewImageResult extends OverviewImageSize {
  sizeFactor: number;
}

const getFinalImageSize = (
  containerSize: ImageContainerSize,
  maxImageSize: OverviewImageSize,
): UseOverviewImageResult => {
  const maxHeight = Math.min(containerSize.height, maxImageSize.height);
  const maxWidth = Math.min(containerSize.width, maxImageSize.width);

  const heightSizeFactor = maxHeight / maxImageSize.height;
  const widthSizeFactor = maxWidth / maxImageSize.width;
  const sizeFactor = Math.min(heightSizeFactor, widthSizeFactor);

  const width = maxImageSize.width * sizeFactor;
  const height = maxImageSize.height * sizeFactor;

  return {
    height,
    width,
    sizeFactor,
  };
};

export const useOverviewImageSize = ({
  containerRect,
}: UseOverviewImageArgs): UseOverviewImageResult => {
  const currentImage = useSelector(currentOverviewImageSelector);

  if (!currentImage || !containerRect) return DEFAULT_OVERVIEW_IMAGE_SIZE;

  const maxImageSize = {
    width: currentImage.width,
    height: currentImage.height,
  };

  return getFinalImageSize(containerRect, maxImageSize);
};
